#include <vtoo/term.h>
#include <vtoo/input.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>

void at_init(void);
void at_exit(void);

int istypeable(int c) {
   return (isprint(c) || isspace(c));
}

int main(int argc, char **argv) {
   at_init();
   set_style(BOLD);
   printf("Unlike a library such as curses, vtoo does not attempt to take control of all console input and output. Instead, vtoo works harmonically with existing input/output functions. For this reason, functions such as scanf and getchar can be used and behave the same way they would in normal circumstances. However, vtoo does include some input functions for single-character inputs that allow you to validate them.");
   printf("\n\nThe first example we will demonstrate is the get_input_charset function, which accepts a string of characters which are allowed inputs, and will continue to wait until one of those keys is pressed, returning this key character as its return value.");
   printf("\n\nTo proceed, press the P, Q, R or S keys on your keyboard, either in upper or lower case.");
   flush();
   char c = get_input_charset("pqrsPQRS");
   clear();
   locate(0,0);
   printf("The character you pressed was: '%c'!\n\n",c);
   printf("The other input control function behaves similarly, except instead of taking a string of valid input characters, it sends the character to a function which, if a nonzero value is returned, will validate the input. If the function returns 0 (false), then vtoo will continue to wait until a valid key is pressed.\n\n");
   printf("The type signature of the function provided to get_input_rule matches the various issomething functions in ctype.h, that is:\n\n int isspace(int c);\n\nTo proceed, press a punctuation key.\n\n");
   flush();
   c = get_input_rule(ispunct);
   clear();
   locate(0,0);
   printf("The character you pressed was: '%c'!\n\n",c);
   printf("That concludes this demonstration. Press a key to quit.");
   flush();
   c = get_input_rule(istypeable);
   return 0;
}

void at_init(void) {
   atexit(at_exit);
   clear();
   term_buffer_full();
   echo_off();
   locate(0,0);
   toggle_cursor();
   flush();
}

void at_exit(void) {
   toggle_cursor();
   term_buffer_line();
   set_style(NO_ATTRIBS);
   clear();
   echo_on();
   locate(0,0);
   flush();
}
