#include <unistd.h>
#include <stdio_ext.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <signal.h>
#include "common.h"
#include "term.h"   
//Here resides horrible platform-dependent code. Would have to be rewritten for other OSs
static int cursor_on = 0;

static struct termios stored_settings;

static void (*atwinch_hdc)(void) = NULL;

void term_buffer_full(void) {
   setvbuf(stdout,NULL,_IOFBF,__fbufsize(stdout));
}

void term_buffer_line(void) {
   setvbuf(stdout,NULL,_IOLBF,__fbufsize(stdout));
}

void term_buffer_none(void) {
   setvbuf(stdout,NULL,_IONBF,__fbufsize(stdout));
}


void locate(int y, int x) {
   printf("\e[%d;%df",y+1,x+1);
}

static void on_winch (int sig) {
   atwinch_hdc();
}

void atwinch (void (fun)(void)) {
   struct sigaction new_action;
   atwinch_hdc = fun;  
   new_action.sa_handler = on_winch;
   sigemptyset (&new_action.sa_mask);
   new_action.sa_flags = 0;
   sigaction(SIGWINCH,&new_action,NULL);
}

void set_style(attributes attribs) {
   printf("\e[0m");   
   if (attribs & BOLD)          printf("\e[1m");
   if (attribs & LOW_INTENSITY) printf("\e[2m");
   if (attribs & EMPHASIS)      printf("\e[3m");
   if (attribs & UNDERLINE)     printf("\e[4m");
   if (attribs & BLINKING)      printf("\e[5m");
   if (attribs & INVERSE)       printf("\e[7m");  
   if (attribs & BLACK)         printf("\e[30m");
   if (attribs & RED)           printf("\e[31m");
   if (attribs & GREEN)         printf("\e[32m");
   if (attribs & YELLOW)        printf("\e[33m");
   if (attribs & BLUE)          printf("\e[34m");
   if (attribs & MAGENTA)       printf("\e[35m");
   if (attribs & CYAN)          printf("\e[36m");
   if (attribs & WHITE)         printf("\e[37m");
   if (attribs & BLACK_BG)      printf("\e[40m");
   if (attribs & RED_BG)        printf("\e[41m");
   if (attribs & GREEN_BG)      printf("\e[42m");
   if (attribs & YELLOW_BG)     printf("\e[43m");
   if (attribs & BLUE_BG)       printf("\e[44m");
   if (attribs & MAGENTA_BG)    printf("\e[45m");
   if (attribs & CYAN_BG)       printf("\e[46m");
   if (attribs & WHITE_BG)      printf("\e[47m");
}

void toggle_cursor(void) {
   cursor_on = !cursor_on;
   if (cursor_on) 
      printf("\e[?25l");
   else
      printf("\e[?25h");
}


void clear(void) {
   printf("\e[2J");   
}

int cols(void) {
   struct winsize mywinsize;
   ioctl(STDOUT_FILENO, TIOCGWINSZ, &mywinsize);
   return mywinsize.ws_col;
}

int rows(void) {
   struct winsize mywinsize;
   ioctl(STDOUT_FILENO, TIOCGWINSZ, &mywinsize);
   return mywinsize.ws_row;
}

void flush(void) {
   fflush(stdout);
}


void echo_off(void) {
   tcgetattr(0,&stored_settings);
   struct termios new_settings = stored_settings;
   new_settings.c_lflag &= (~(ECHO | ICANON));
   tcsetattr(0,TCSANOW,&new_settings);
}

void echo_on(void) {
   tcsetattr(0,TCSANOW,&stored_settings);
}

